package com.sun.beans.decoder;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import com.sun.beans.finder.MethodFinder;
import java.beans.IndexedPropertyDescriptor;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.beans.IntrospectionException;
import java.lang.reflect.Array;
import sun.reflect.misc.MethodUtil;

final class PropertyElementHandler extends AccessorElementHandler
{
    static final String GETTER = "get";
    static final String SETTER = "set";
    private Integer index;
    
    PropertyElementHandler() {
    }
    
    @Override
    public void addAttribute(String s, String s2) {
        if (s.equals("index")) {
            this.index = Integer.valueOf(s2);
        }
        else {
            super.addAttribute(s, s2);
        }
    }
    
    @Override
    protected boolean isArgument() {
        return false;
    }
    
    @Override
    protected Object getValue(String s) {
        try {
            return getPropertyValue(this.getContextBean(), s, this.index);
        }
        catch (Exception ex) {
            this.getOwner().handleException(ex);
            return null;
        }
    }
    
    @Override
    protected void setValue(String s, Object o) {
        try {
            setPropertyValue(this.getContextBean(), s, this.index, o);
        }
        catch (Exception ex) {
            this.getOwner().handleException(ex);
        }
    }
    
    private static Object getPropertyValue(Object o, String s, Integer n) throws IllegalAccessException, IntrospectionException, InvocationTargetException, NoSuchMethodException {
        Class<Object> class1 = o.getClass();
        if (n == null) {
            return MethodUtil.invoke(findGetter(class1, s, new Class[0]), o, new Object[0]);
        }
        if (class1.isArray() && s == null) {
            return Array.get(o, (int)n);
        }
        return MethodUtil.invoke(findGetter(class1, s, new Class[] { Integer.TYPE }), o, new Object[] { n });
    }
    
    private static void setPropertyValue(Object o, String s, Integer n, Object o2) throws IllegalAccessException, IntrospectionException, InvocationTargetException, NoSuchMethodException {
        Class<Object> class1 = o.getClass();
        Class<Object> clazz = (o2 != null) ? o2.getClass() : null;
        if (n == null) {
            MethodUtil.invoke(findSetter(class1, s, new Class[] { clazz }), o, new Object[] { o2 });
        }
        else if (class1.isArray() && s == null) {
            Array.set(o, (int)n, o2);
        }
        else {
            MethodUtil.invoke(findSetter(class1, s, new Class[] { Integer.TYPE, clazz }), o, new Object[] { n, o2 });
        }
    }
    
    private static Method findGetter(Class<?> type, String str, Class<?>... args) throws IntrospectionException, NoSuchMethodException {
        if (str == null) {
            return MethodFinder.findInstanceMethod(type, "get", args);
        }
        PropertyDescriptor property = getProperty(type, str);
        if (args.length == 0) {
            Method readMethod = property.getReadMethod();
            if (readMethod != null) {
                return readMethod;
            }
        }
        else if ((property instanceof IndexedPropertyDescriptor)) {
            Method indexedReadMethod = ((IndexedPropertyDescriptor)property).getIndexedReadMethod();
            if (indexedReadMethod != null) {
                return indexedReadMethod;
            }
        }
        throw new IntrospectionException("Could not find getter for the " + str + " property");
    }
    
    private static Method findSetter(Class<?> type, String str, Class<?>... args) throws IntrospectionException, NoSuchMethodException {
        if (str == null) {
            return MethodFinder.findInstanceMethod(type, "set", args);
        }
        PropertyDescriptor property = getProperty(type, str);
        if (args.length == 1) {
            Method writeMethod = property.getWriteMethod();
            if (writeMethod != null) {
                return writeMethod;
            }
        }
        else if ((property instanceof IndexedPropertyDescriptor)) {
            Method indexedWriteMethod = ((IndexedPropertyDescriptor)property).getIndexedWriteMethod();
            if (indexedWriteMethod != null) {
                return indexedWriteMethod;
            }
        }
        throw new IntrospectionException("Could not find setter for the " + str + " property");
    }
    
    private static PropertyDescriptor getProperty(Class<?> beanClass, String str) throws IntrospectionException {
        PropertyDescriptor[] propertyDescriptors = Introspector.getBeanInfo(beanClass).getPropertyDescriptors();
        int length = propertyDescriptors.length;
        int i = 0;
        while (i < length) {
            PropertyDescriptor propertyDescriptor = propertyDescriptors[i];
            if (str.equals(propertyDescriptor.getName())) {
                return propertyDescriptor;
            }
            ++i;
        }
        throw new IntrospectionException("Could not find the " + str + " property descriptor");
    }
}
